@@ -400,7 +400,9 @@ static sasl_security_properties_t *make_secprops(int min,int max)
 }
 
 static int acap_conn_do_auth(acap_conn_t *conn,
-			     const char *user, const char *mechs)
+			     const char *user, 
+			     const char *mechs,
+			     sasl_callback_t *cb)
 {
     struct sockaddr_in sin;
     sasl_security_properties_t *secprops=NULL;
@@ -409,56 +411,90 @@ static int acap_conn_do_auth(acap_conn_t *conn,
     unsigned int len;
     int r;
     acap_cmd_t *cmd;
-    sasl_callback_t *cb = 
-	(sasl_callback_t *) malloc(2 * sizeof(sasl_callback_t));
+    sasl_callback_t *mycb;
+    int didusercb, i;
 
-    cb[0].id = SASL_CB_USER;
-    cb[0].proc = &mysasl_simple_cb;
-    cb[0].context = (void *) user;
-    cb[1].id = SASL_CB_LIST_END;
-    cb[1].proc = NULL;
-    cb[1].context = NULL;
+    /* copy the sasl callbacks, to ours, add the CB_USER */
+    i = 0;
+    if (cb) {
+	while (cb[i].id != SASL_CB_LIST_END) i++;
+    }
 
-    r = sasl_client_new("acap", conn->host, cb, 
+    i += 2; /* one for SASL_CB_LIST_END, one for SASL_CB_USER */
+    mycb = (sasl_callback_t *) malloc(i * sizeof(sasl_callback_t));
+
+    i = 0;
+    didusercb = 0;
+    if (cb) {
+	while (cb[i].id != SASL_CB_LIST_END) {
+	    if (cb[i].id == SASL_CB_USER) {
+		/* override */
+		mycb[i].id = SASL_CB_USER;
+		mycb[i].proc = &mysasl_simple_cb;
+		mycb[i].context = (void *) user;
+		
+		didusercb = 1;
+	    } else {
+		/* copy */
+		mycb[i].id = cb[i].id;
+		mycb[i].proc = cb[i].proc;
+		mycb[i].context = cb[i].context;
+	    }
+	    
+	    i++;
+	}
+    }
+    if (!didusercb) {		/* add on user callback */
+	mycb[i].id = SASL_CB_USER;
+	mycb[i].proc = &mysasl_simple_cb;
+	mycb[i].context = (void *) user;
+
+	i++;
+    }
+    /* add SASL_CB_LIST_END */
+    mycb[i].id = SASL_CB_LIST_END;
+    mycb[i].proc = NULL;
+    mycb[i].context = NULL;
+
+    r = sasl_client_new("acap", conn->host, mycb, 
 			SASL_SECURITY_LAYER, &conn->sasl);
     if (r != SASL_OK || !conn->sasl) {
 	conn->sasl_error = r;
-	free(cb);
+	free(mycb);
 	return ACAP_NO_AUTHENTICATION;
     }
 
     len = sizeof(sin);
     if (getpeername(conn->sock, (struct sockaddr *) &sin, &len)) {
-	free(cb);
+	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
     sasl_setprop(conn->sasl, SASL_IP_REMOTE, &sin);
     if (r != SASL_OK || !conn->sasl) {
 	conn->sasl_error = r;
-	free(cb);
+	free(mycb);
 	return ACAP_NO_AUTHENTICATION;
     }
 
     len = sizeof(sin);
     if (getsockname(conn->sock, (struct sockaddr *) &sin, &len)) {
-	free(cb);
+	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
     sasl_setprop(conn->sasl, SASL_IP_LOCAL, &sin);
     if (r != SASL_OK || !conn->sasl) {
 	conn->sasl_error = r;
-	free(cb);
+	free(mycb);
 	return ACAP_NO_AUTHENTICATION;
     }
 
     secprops = make_secprops(0, 0); /* xxx */
-    if (secprops != NULL)
-    {
+    if (secprops != NULL) {
 	sasl_setprop(conn->sasl, SASL_SEC_PROPS, secprops);
 	free(secprops);
     }
 
-    r = sasl_client_start(conn->sasl, "KERBEROS_V4",
+    r = sasl_client_start(conn->sasl, mechs,
 			  NULL, NULL,
 			  &str, &len, &mech);
     if (r == SASL_OK || r == SASL_CONTINUE) {
@@ -477,7 +513,7 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 	}
     } else {
 	conn->sasl_error = r;
-	free(cb);
+	free(mycb);
 	return ACAP_NO_AUTHENTICATION;
     }
 
@@ -489,7 +525,7 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 
     r = acap_process_on_command(conn, cmd, NULL);
     if (r != ACAP_OK) {
-	free(cb);
+	free(mycb);
 	return r;
     }
 
@@ -497,15 +533,15 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 	prot_setsasl(conn->pin,  conn->sasl);
 	prot_setsasl(conn->pout, conn->sasl);
 
-	free(cb);
+	free(mycb);
 	return ACAP_OK;
     } else {
-	free(cb);
+	free(mycb);
 	return ACAP_NO_AUTHENTICATION;
     }
 }
 
-int acap_conn_connect(const char *url, acap_conn_t **ret)
+int acap_conn_connect(const char *url, sasl_callback_t *cb, acap_conn_t **ret)
 {
     char *user = NULL, *mech = NULL;
     char *host = NULL, *port = NULL;
@@ -539,7 +575,7 @@ int acap_conn_connect(const char *url, acap_conn_t **ret)
 	return r;
     }
 
-    r = acap_conn_do_auth(conn, user, mech);
+    r = acap_conn_do_auth(conn, user, mech, cb);
     if (user) free(user);
     if (mech) free(mech);
     if (r != ACAP_OK) {
@@ -589,6 +625,7 @@ static int send_quoted_p(int l, char *a)
     if (l > 128) return 0;
 
     for (i = 0; i < l; i++) {
+	if (a[i] == '*') continue;
 	if (!isprint((int) a[i])) return 0;
 	if (a[i] == '"') return 0;
 	if (a[i] == '\\') return 0;
@@ -897,10 +934,8 @@ void acap_entry_free(acap_entry_t *e)
 
     /* free it */
     free(e->name);
-    /* free the attributes */
-    sforeach(e->attrs, (void (*)(const void *))acap_attribute_free);
-    /* free the list */
-    skiplist_free(e->attrs);
+    /* free the attributes & free the list */
+    skiplist_freeeach(e->attrs, (void (*)(const void *))acap_attribute_free);
     free(e);
 }
 
@@ -2189,10 +2224,12 @@ int acap_store_entry(acap_conn_t *conn,
     acap_attribute_t *attr;
     char *modtime = NULL;
     skipnode *node;
+    int first;
 
     if (!conn) return ACAP_NO_CONNECTION;
     if ((flags & ACAP_STORE_INITIAL) && 
 	(flags & ACAP_STORE_FORCE)) return ACAP_BAD_PARAM;
+    if (!entry) return ACAP_BAD_PARAM;
 
     if (flags & ACAP_STORE_INITIAL) {
 	modtime = initial_time;
@@ -2211,12 +2248,19 @@ int acap_store_entry(acap_conn_t *conn,
 	prot_printf(conn->pout, "UNCHANGEDSINCE \"%s\" ", modtime);
     }
 
-    attr = sfirst(entry->attrs, &node);
-    while (attr != NULL) {
+    first = 1;
+    for (attr = sfirst(entry->attrs, &node); attr != NULL; attr = snext(&node))
+    {
 	int l;
 
 	if (!strcmp(attr->name, "modtime")) continue;
 	if (!strcmp(attr->name, "entry")) continue;
+
+	if (!first) {
+	    prot_putc(' ', conn->pout);
+	}
+	first = 0;
+
 	l = strlen(attr->name);
 	if (send_quoted_p(l, attr->name)) {
 	    prot_putc('"', conn->pout);
@@ -2240,6 +2284,7 @@ int acap_store_entry(acap_conn_t *conn,
 		if (v->next) { prot_putc(' ', conn->pout); }
 	    }
 	    prot_printf(conn->pout, "))");
+	    break;
 
 	case ACAP_VALUE_NIL:
 	    prot_printf(conn->pout, " NIL");
@@ -2249,9 +2294,6 @@ int acap_store_entry(acap_conn_t *conn,
 	    prot_printf(conn->pout, " DEFAULT");
 	    break;
 	}
-
-	attr = snext(&node);
-	if (attr) prot_putc(' ', conn->pout);
     }
 
     prot_printf(conn->pout, ")\r\n");
