@@ -1,33 +1,52 @@
 /* arbitron.c -- program to report readership statistics
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Copyright (c) 1998, 2000 Carnegie Mellon University.  All rights reserved.
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes or
- * benefits requires specific, additional permission from:
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
  *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: arbitron.c,v 1.16 1999/08/31 18:39:19 leg Exp $ */
+/* $Id: arbitron.c,v 1.21 2000/05/23 20:52:12 robeson Exp $ */
 
+#include <config.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -36,13 +55,17 @@
 #include <netinet/in.h>
 #include <sys/stat.h>
 #include <com_err.h>
+#include <time.h>
 
 #include "assert.h"
-#include "config.h"
+#include "imapconf.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
+#include "mboxlist.h"
+#include "convert_code.h"
+#include "seen.h"
 
 extern int errno;
 extern int optind;
@@ -52,18 +75,19 @@ int code = 0;
 
 time_t report_time, prune_time = 0;
 
+/* forward declarations */
+void usage(void);
 int do_mailbox();
+int arbitron(char *name);
 
 struct arbitronargs {
     char *name;
     unsigned read_count;
 };
 
-main(argc, argv)
-int argc;
-char **argv;
+int main(int argc,char **argv)
 {
-    int opt, i;
+    int opt;
     int report_days = 30;
     int prune_months = 0;
     char pattern[MAX_MAILBOX_NAME+1];
@@ -101,9 +125,11 @@ char **argv;
     mboxlist_findall(pattern, 1, 0, 0, do_mailbox, NULL);
 
     exit(code);
+
+    return -1; /* never reaches */
 }
 
-usage()
+void usage(void)
 {
     fprintf(stderr, "usage: arbitron [-d days] [-p months] [mboxpattern]\n");
     exit(EC_USAGE);
@@ -148,9 +174,7 @@ const char *line;
     return 0;
 }
 
-int 
-arbitron(name)
-char *name;
+int arbitron(char *name)
 {
     int r;
     struct mailbox mailbox;
