@@ -1,34 +1,52 @@
 /* fud.c -- long-lived finger information provider
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- * Copyright 1998 Carnegie Mellon University
- * 
- * No warranties, either expressed or implied, are made regarding the
- * operation, use, or results of the software.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission to use, copy, modify and distribute this software and its
- * documentation is hereby granted for non-commercial purposes only
- * provided that this copyright notice appears in all copies and in
- * supporting documentation.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * Permission is also granted to Internet Service Providers and others
- * entities to use the software for internal purposes.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
  *
- * The distribution, modification or sale of a product which uses or is
- * based on the software, in whole or in part, for commercial purposes
- * or
- * benefits requires specific, additional permission from:
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
  *
- *  Office of Technology Transfer
- *  Carnegie Mellon University
- *  5000 Forbes Avenue
- *  Pittsburgh, PA  15213-3890
- *  (412) 268-4387, fax: (412) 268-7395
- *  tech-transfer@andrew.cmu.edu
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
  *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+
  */
 
-/* $Id: fud.c,v 1.9 1999/04/08 21:04:22 tjs Exp $ */
+#include <config.h>
 
+/* $Id: fud.c,v 1.15 2000/05/23 20:52:15 robeson Exp $ */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -45,12 +63,15 @@
 #include <pwd.h>
 
 #include "assert.h"
-#include "config.h"
+#include "imapconf.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
 #include "acl.h"
+#include "seen.h"
+#include "mboxname.h"
+#include "map.h"
 
 #define REQ_OK		0
 #define REQ_DENY	1
@@ -60,44 +81,19 @@ extern int errno;
 extern int optind;
 extern char *optarg;
 
-void send_reply P((struct sockaddr_in sfrom, int status,
-		   char *user, char *mbox, int numrecent, time_t lastread,
-		   time_t lastarrived));
+/* forward decls */
+int handle_request(char *who, char *name, struct sockaddr_in sfrom);
+
+void send_reply(struct sockaddr_in sfrom, int status,
+		char *user, char *mbox, int numrecent, time_t lastread,
+		time_t lastarrived);
 
 int code = 0;
 int soc;
 
 char who[16];
 
-int
-main(argc, argv)
-int argc; 
-char **argv;
-{
-    int port, r;
-   
-
-    r = 0; /* to shut up lint/gcc */
-
-    config_init("fud");
-
-    if(geteuid() == 0)
-        fatal("must run as the Cyrus user", EC_USAGE);
-    r = init_network(port);
-    signal(SIGHUP,SIG_IGN);
-
-    if (r)
-        fatal("unable to configure network port", EC_OSERR);
-    
-    begin_handling();
-
-    exit(code);
-}
-
-
-int
-init_network(port)
-int port;
+int init_network(int port)
 {
     soc = 0;	/* inetd has handed us the port as stdin */
     return(0);
@@ -105,8 +101,7 @@ int port;
 
 #define MAXLOGNAME 16		/* should find out for real */
 
-int
-begin_handling()
+int begin_handling(void)
 {
         struct sockaddr_in  sfrom;
         int sfromsiz = sizeof(sfrom);
@@ -133,17 +128,43 @@ begin_handling()
                     continue;
             }
             q = buf + off + 1;
-            strncpy(mbox,q,(r - (off + 1)  < MAX_MAILBOX_NAME) ? r - (off + 1) : MAX_MAILBOX_NAME);
+            strncpy(mbox,q,(r - (off + 1)  < MAX_MAILBOX_NAME) ? 
+		             r - (off + 1) : MAX_MAILBOX_NAME);
 
             handle_request(username,mbox,sfrom);
         }
 }
 
-int 
-handle_request(who,name,sfrom)
-char *who;
-char *name;
-struct sockaddr_in sfrom;
+
+
+int main(int argc, char **argv)
+{
+    int port = 0;
+    int r;
+   
+    r = 0; /* to shut up lint/gcc */
+
+    config_init("fud");
+
+    if(geteuid() == 0)
+        fatal("must run as the Cyrus user", EC_USAGE);
+    r = init_network(port);
+    signal(SIGHUP,SIG_IGN);
+
+    if (r)
+        fatal("unable to configure network port", EC_OSERR);
+    
+    begin_handling();
+
+    exit(code);
+}
+
+
+
+
+
+
+int handle_request(char *who,char *name,struct sockaddr_in sfrom)
 {
     int r;
     struct mailbox mailbox;
@@ -184,7 +205,7 @@ struct sockaddr_in sfrom;
 
     r = seen_open(&mailbox, who, &seendb);
     if (r) return r;
-    r = seen_lockread(seendb, &lastread, &lastuid, &lastarrived, &seenuids);
+    r = seen_read(seendb, &lastread, &lastuid, &lastarrived, &seenuids);
     seen_close(seendb);
     if (r) return r;
     
