@@ -1,6 +1,6 @@
 /* lmtpd.c -- Program to deliver mail to a mailbox
  *
- * $Id: lmtpd.c,v 1.36 2000/06/09 02:44:50 leg Exp $
+ * $Id: lmtpd.c,v 1.45 2000/08/04 18:38:33 leg Exp $
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,7 @@
  *
  */
 
-/*static char _rcsid[] = "$Id: lmtpd.c,v 1.36 2000/06/09 02:44:50 leg Exp $";*/
+/*static char _rcsid[] = "$Id: lmtpd.c,v 1.45 2000/08/04 18:38:33 leg Exp $";*/
 
 #include <config.h>
 
@@ -67,6 +67,7 @@
 
 #include <pwd.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #endif /* USE_SIEVE */
 
 #include <netdb.h>
@@ -386,6 +387,11 @@ int getenvelope(void *mc, const char *field, const char ***contents)
 	mydata->temp[0] = msg_getrcptall(m, mydata->cur_rcpt);
 	mydata->temp[1] = NULL;
 	return SIEVE_OK;
+    } else if (!strcasecmp(field, "auth") && mydata->authuser) {
+	*contents = mydata->temp;
+	mydata->temp[0] = mydata->authuser;
+	mydata->temp[1] = NULL;
+	return SIEVE_OK;
     } else {
 	*contents = NULL;
 	return SIEVE_FAIL;
@@ -431,6 +437,34 @@ pid_t open_sendmail(const char *argv[], FILE **sm)
     return p;
 }
 
+/* sendmail_errstr.  create a descriptive message given 'sm_stat': 
+   the exit code from wait() from sendmail.
+
+   not thread safe, but probably ok */
+static char *sendmail_errstr(int sm_stat)
+{
+    static char errstr[200];
+
+    if (WIFEXITED(sm_stat)) {
+	snprintf(errstr, sizeof errstr,
+		 "Sendmail process terminated normally, exit status %d\n",
+		 WEXITSTATUS(sm_stat));
+    } else if (WIFSIGNALED(sm_stat)) {
+	snprintf(errstr, sizeof errstr,
+		"Sendmail process terminated abnormally, signal = %d %s\n",
+		WTERMSIG(sm_stat), 
+		WCOREDUMP(sm_stat) ? " -- core file generated" : "");
+    } else if (WIFSTOPPED(sm_stat)) {
+	snprintf(errstr, sizeof errstr,
+		 "Sendmail process stopped, signal = %d\n",
+		WTERMSIG(sm_stat));
+    } else {
+	return NULL;
+    }
+    
+    return errstr;
+}
+
 int send_rejection(const char *origid,
 		   const char *rejto,
 		   const char *origreceip, 
@@ -439,19 +473,22 @@ int send_rejection(const char *origid,
 		   struct protstream *file)
 {
     FILE *sm;
-    const char *smbuf[3];
+    const char *smbuf[6];
     char buf[8192], *namebuf;
-    int i;
+    int i, sm_stat;
     time_t t;
     struct tm *tm;
     long gmtoff;
     int gmtnegative = 0;
-    pid_t p;
+    pid_t sm_pid, p;
 
     smbuf[0] = "sendmail";
-    smbuf[1] = rejto;
-    smbuf[2] = NULL;
-    p = open_sendmail(smbuf, &sm);
+    smbuf[1] = "-f";
+    smbuf[2] = "<>";
+    smbuf[3] = "--";
+    smbuf[4] = rejto;
+    smbuf[5] = NULL;
+    sm_pid = open_sendmail(smbuf, &sm);
     if (sm == NULL) {
 	return -1;
     }
@@ -520,18 +557,18 @@ int send_rejection(const char *origid,
     fprintf(sm, "--%d/%s\r\n", (int) p, config_servername);
 
     fclose(sm);
-    waitpid(p, &i, 0);
+    while (waitpid(sm_pid, &sm_stat, 0) < 0);
 
-    return (i == 0 ? SIEVE_OK : SIEVE_FAIL); /* sendmail exit value */
+    return sm_stat;	/* sendmail exit value */
 }
 
 int send_forward(char *forwardto, char *return_path, struct protstream *file)
 {
     FILE *sm;
     const char *smbuf[6];
-    int i;
+    int i, sm_stat;
     char buf[1024];
-    pid_t p;
+    pid_t sm_pid;
 
     smbuf[0] = "sendmail";
     if (return_path != NULL) {
@@ -539,12 +576,12 @@ int send_forward(char *forwardto, char *return_path, struct protstream *file)
 	smbuf[2] = return_path;
     } else {
 	smbuf[1] = "-f";
-	smbuf[2] = POSTMASTER;
+	smbuf[2] = "<>";
     }
     smbuf[3] = "--";
     smbuf[4] = forwardto;
     smbuf[5] = NULL;
-    p = open_sendmail(smbuf, &sm);
+    sm_pid = open_sendmail(smbuf, &sm);
 	
     if (sm == NULL) {
 	return -1;
@@ -557,9 +594,9 @@ int send_forward(char *forwardto, char *return_path, struct protstream *file)
     }
 
     fclose(sm);
-    waitpid(p, &i, 0);
+    while (waitpid(sm_pid, &sm_stat, 0) < 0);
 
-    return (i == 0 ? SIEVE_OK : SIEVE_FAIL); /* sendmail exit value */
+    return sm_stat;	/* sendmail exit value */
 }
 
 
@@ -577,7 +614,7 @@ int sieve_redirect(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	if (res == -1) {
 	    *errmsg = "Could not spawn sendmail process";
 	} else {
-	    *errmsg = "Sendmail error";
+	    *errmsg = sendmail_errstr(res);
 	}
 	return SIEVE_FAIL;
     }
@@ -609,7 +646,7 @@ int sieve_reject(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     }
     
     body = msg_getheader(md, "original-recipient");
-    origreceip = body[0];
+    origreceip = body ? body[0] : NULL;
     if ((res = send_rejection(md->id, md->return_path, 
 			      origreceip, sd->username,
 			      rc->msg, md->data)) == 0) {
@@ -619,7 +656,7 @@ int sieve_reject(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	if (res == -1) {
 	    *errmsg = "Could not spawn sendmail process";
 	} else {
-	    *errmsg = "Sendmail error";
+	    *errmsg = sendmail_errstr(res);
 	}
 	return SIEVE_FAIL;
     }
@@ -758,22 +795,23 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     FILE *sm;
     const char *smbuf[6];
     char outmsgid[8192], *sievedb;
-    int i, sl;
+    int i, sl, sm_stat;
     struct tm *tm;
     long tz;
     int tznegative = 0;
     time_t t;
-    pid_t p;
+    pid_t sm_pid, p;
     sieve_send_response_context_t *src = (sieve_send_response_context_t *) ac;
+    message_data_t *m = (message_data_t *) mc;
     script_data_t *sdata = (script_data_t *) sc;
 
     smbuf[0] = "sendmail";
     smbuf[1] = "-f";
-    smbuf[2] = POSTMASTER;
+    smbuf[2] = "<>";
     smbuf[3] = "--";
     smbuf[4] = src->addr;
     smbuf[5] = NULL;
-    p = open_sendmail(smbuf, &sm);
+    sm_pid = open_sendmail(smbuf, &sm);
     if (sm == NULL) {
 	*errmsg = "Could not spawn sendmail process";
 	return -1;
@@ -810,6 +848,7 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	    break;
 	}
     fprintf(sm, "Subject: %s\r\n", src->subj);
+    fprintf(sm, "In-Reply-To: %s\r\n", m->id);
     fprintf(sm, "Auto-Submitted: auto-generated (vacation)\r\n");
     if (src->mime) {
 	fprintf(sm, "MIME-Version: 1.0\r\n");
@@ -827,9 +866,9 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	fprintf(sm, "\r\n--%d/%s\r\n", (int) p, config_servername);
     }
     fclose(sm);
-    waitpid(p, &i, 0);
+    while (waitpid(sm_pid, &sm_stat, 0) < 0);
 
-    if (i == 0) { /* i is sendmail exit value */
+    if (sm_stat == 0) { /* sendmail exit value */
 	sievedb = make_sieve_db(sdata->username);
 
 	duplicate_mark(outmsgid, strlen(outmsgid), 
@@ -839,7 +878,7 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 
 	return SIEVE_OK;
     } else {
-	*errmsg = "Sendmail error";
+	*errmsg = sendmail_errstr(sm_stat);
 	return SIEVE_FAIL;
     }
 }
@@ -856,7 +895,7 @@ sieve_vacation_t vacation = {
 static char *markflags[] = { "\\flagged" };
 static sieve_imapflags_t mark = { markflags, 1 };
 
-int sieve_parse_error_handler(int lineno, char *msg, void *ic, void *sc)
+int sieve_parse_error_handler(int lineno, const char *msg, void *ic, void *sc)
 {
     script_data_t *sd = (script_data_t *) sc;
     
@@ -866,7 +905,7 @@ int sieve_parse_error_handler(int lineno, char *msg, void *ic, void *sc)
     return SIEVE_OK;
 }
 
-int sieve_execute_error_handler(char *msg, void *ic, void *sc, void *mc)
+int sieve_execute_error_handler(const char *msg, void *ic, void *sc, void *mc)
 {
     script_data_t *sd = (script_data_t *) sc;
     message_data_t *md = ((mydata_t *) mc)->m;
@@ -913,7 +952,6 @@ static void setup_sieve(void)
 	fatal("sieve_register_keep()", EC_TEMPFAIL);
     }
     res = sieve_register_imapflags(sieve_interp, &mark);
-
     if (res != SIEVE_OK) {
 	syslog(LOG_ERR, "sieve_register_imapflags() returns %d\n", res);
 	fatal("sieve_register_imapflags()", EC_TEMPFAIL);
@@ -1055,19 +1093,17 @@ int deliver_mailbox(struct protstream *msg,
 	return 0;
     }
     r = append_setup(&as, namebuf, MAILBOX_FORMAT_NORMAL,
-		     authstate, acloverride ? 0 : ACL_POST, 
+		     NULL, authstate, acloverride ? 0 : ACL_POST, 
 		     quotaoverride ? -1 : 0);
 
     if (!r) {
 	prot_rewind(msg);
 	if (singleinstance && stage) {
 	    r = append_fromstage(&as, msg, size, now, 
-				 (const char **) flag, nflags,
-				 user, stage);
+				 (const char **) flag, nflags, stage);
 	} else {
 	    r = append_fromstream(&as, msg, size, now, 
-				  (const char **) flag, nflags,
-				  user);
+				  (const char **) flag, nflags);
 	}
 	if (!r) append_commit(&as, NULL, NULL, NULL);
 	else append_abort(&as);
@@ -1128,6 +1164,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    FILE *f = sieve_find_script(rcpt);
 	    char namebuf[MAX_MAILBOX_PATH];
 
+#ifdef USE_SIEVE
 	    if (f != NULL) {
 		script_data_t *sdata = NULL;
 		sieve_script_t *s = NULL;
@@ -1140,7 +1177,8 @@ int deliver(message_data_t *msgdata, char *authuser,
 
 		/* slap the mailboxname back on so we hash the envelope & id
 		   when we figure out whether or not to keep the message */
-		snprintf(namebuf, sizeof(namebuf), "%s+%s", rcpt, plus);
+		snprintf(namebuf, sizeof(namebuf), "%s+%s", rcpt,
+			 plus ? plus : "");
 		
 		/* is this the first time we've sieved the message? */
 		if (msgdata->id) {
@@ -1179,6 +1217,9 @@ int deliver(message_data_t *msgdata, char *authuser,
 		/* no sieve script */
 		r = 1; /* do normal delivery actions */
 	    }
+#else /* USE_SIEVE */
+	    r = 1;		/* normal delivery */
+#endif /* USE_SIEVE */
 
 	    if (r && plus &&
 		strlen(rcpt) + strlen(plus) + 30 <= MAX_MAILBOX_PATH) {
