@@ -1,27 +1,47 @@
 /* imclient.c -- Streaming IMxP client library
- $Id: imclient.c,v 1.45 1999/12/23 21:03:19 leg Exp $
+ $Id: imclient.c,v 1.51 2000/05/24 05:35:43 leg Exp $
  
- #        Copyright 1998 by Carnegie Mellon University
- #
- #                      All Rights Reserved
- #
- # Permission to use, copy, modify, and distribute this software and its
- # documentation for any purpose and without fee is hereby granted,
- # provided that the above copyright notice appear in all copies and that
- # both that copyright notice and this permission notice appear in
- # supporting documentation, and that the name of CMU not be
- # used in advertising or publicity pertaining to distribution of the
- # software without specific, written prior permission.
- #
- # CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
- # ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
- # CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
- # ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- # WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- # ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- # SOFTWARE.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
  *
  */
+#include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -175,6 +195,8 @@ static int imclient_decodebase64 P((char *input));
 static sasl_callback_t callbacks[] = {
   {
     SASL_CB_USER, NULL, NULL
+  }, {
+    SASL_CB_GETREALM, NULL, NULL
   }, {
     SASL_CB_AUTHNAME, NULL, NULL
   }, {
@@ -641,13 +663,16 @@ int len;
     char *plainbuf;
     int plainlen;
     int freeplain;
+    int result;
     
     if (imclient->saslcompleted == 1) {
 	/* decrypt what we have */
-	if (sasl_decode(imclient->saslconn, buf, len,
-			&plainbuf, &plainlen) != SASL_OK) {
+	if ((result = sasl_decode(imclient->saslconn, buf, len,
+			&plainbuf, &plainlen)) != SASL_OK) {
 	    (void) shutdown(imclient->fd, 0);
 	}
+
+	if (plainlen == 0) return;
 	freeplain = 1;
     } else {
 	plainbuf = buf;
@@ -746,7 +771,7 @@ int len;
 	}
 	else {
 	    replytag = 0;
-	    while (isdigit(*p)) replytag = replytag * 10 + *p++ - '0';
+	    while (isdigit((int) *p)) replytag = replytag * 10 + *p++ - '0';
 	    if (*p++ != ' ') {
 		/* XXX Got junk from the server */
 		/* Start parsing the next reply */
@@ -756,9 +781,9 @@ int len;
 	}
 
 	/* parse num, if there */
-	if (replytag == 0 && isdigit(*p)) {
+	if (replytag == 0 && isdigit((int) *p)) {
 	    reply.msgno = 0;
-	    while (isdigit(*p)) reply.msgno = reply.msgno * 10 + *p++ - '0';
+	    while (isdigit((int) *p)) reply.msgno = reply.msgno * 10 + *p++ - '0';
 	    if (*p++ != ' ') {
 		/* XXX Got junk from the server */
 		/* Start parsing the next reply */
@@ -795,16 +820,16 @@ int len;
 
 	    /* Scan back and see if the end of the line introduces a literal */
 	    if (!iscompletion && endreply[-1] == '\r' && endreply[-2] == '}' &&
-		isdigit(endreply[-3])) {
+		isdigit((int) endreply[-3])) {
 		p = endreply - 4;
-		while (p > imclient->replystart && isdigit(*p)) p--;
+		while (p > imclient->replystart && isdigit((int) *p)) p--;
 		if (p > imclient->replystart + 2 && *p == '{' &&
 		    charclass[(unsigned char)p[-1]] != 2) {
 
 		    /* Parse the size of the literal */
 		    literallen = 0;
 		    p++;
-		    while (isdigit(*p)) literallen = literallen*10 + *p++ -'0';
+		    while (isdigit((int) *p)) literallen = literallen*10 + *p++ -'0';
 
 		    /* Do a continue to read literal & following line */
 		    imclient->replyliteralleft = literallen;
@@ -862,16 +887,16 @@ int len;
 	/* Scan back and see if the end of the line introduces a literal */
 	if (!(imclient->callback[keywordindex].flags & CALLBACK_NOLITERAL)) {
 	    if (endreply[-1] == '\r' && endreply[-2] == '}' &&
-		isdigit(endreply[-3])) {
+		isdigit((int) endreply[-3])) {
 		p = endreply - 4;
-		while (p > imclient->replystart && isdigit(*p)) p--;
+		while (p > imclient->replystart && isdigit((int) *p)) p--;
 		if (p > imclient->replystart + 2 && *p == '{' &&
 		    charclass[(unsigned char)p[-1]] != 2) {
 
 		    /* Parse the size of the literal */
 		    literallen = 0;
 		    p++;
-		    while (isdigit(*p)) literallen = literallen*10 + *p++ -'0';
+		    while (isdigit((int) *p)) literallen = literallen*10 + *p++ -'0';
 
 		    /* Do a continue to read literal & following line */
 		    imclient->replyliteralleft = literallen;
@@ -950,6 +975,8 @@ struct imclient *imclient;
     int n;
     int writelen;
     fd_set rfds, wfds;
+    FD_ZERO(&rfds);
+    FD_ZERO(&wfds);
 
     for (;;) {
 	writelen = imclient->outptr - imclient->outstart;
@@ -1237,7 +1264,7 @@ int imclient_authenticate(struct imclient *imclient,
     /* stop looping on command completion */
     if (!imclient->readytxt) break;
 
-    if (isspace(*imclient->readytxt)) {
+    if (isspace((int) *imclient->readytxt)) {
 	inlen = 0;
     } else {
 	inlen = imclient_decodebase64(imclient->readytxt);
@@ -1411,20 +1438,8 @@ static int verify_error = X509_V_OK;
 static int do_dump = 1;
 
 #define CCERT_BUFSIZ 256
-static char peer_subject[CCERT_BUFSIZ];
-static char peer_issuer[CCERT_BUFSIZ];
 static char peer_CN[CCERT_BUFSIZ];
 static char issuer_CN[CCERT_BUFSIZ];
-static unsigned char md[EVP_MAX_MD_SIZE];
-static char fingerprint[EVP_MAX_MD_SIZE * 3];
-
-char   *tls_peer_CN = "";
-char   *tls_issuer_CN = NULL;
-
-char   *tls_protocol = NULL;
-const char   *tls_cipher_name = NULL;
-int	tls_cipher_usebits = 0;
-int	tls_cipher_algbits = 0;
 
 /*
   * Set up the cert things on the server side. We do need both the
@@ -1714,13 +1729,15 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 	return (ret);
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
-		 argi, ret, ret);
+	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))\n", (unsigned int) bio, 
+	       (unsigned long) argp,
+	       argi, ret, (unsigned int) ret);
 	tls_dump(argp, (int) ret);
 	return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
-		 argi, ret, ret);
+	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))\n", (unsigned int) bio, 
+	       (unsigned long) argp,
+	       argi, ret, (unsigned int) ret);
 	tls_dump(argp, (int) ret);
     }
     return (ret);
@@ -1730,11 +1747,15 @@ int tls_start_clienttls(struct imclient *imclient,
 			int *layer, char **authid, int fd)
 {
     int     sts;
-    int     j;
-    unsigned int n;
     SSL_SESSION *session;
     SSL_CIPHER *cipher;
     X509   *peer;
+    const char *tls_protocol = NULL;
+    const char *tls_cipher_name = NULL;
+    int tls_cipher_usebits = 0;
+    int tls_cipher_algbits = 0;
+    char *tls_peer_CN = "";
+    char *tls_issuer_CN = NULL;
 
     if (imclient->tls_conn == NULL) {
 	imclient->tls_conn = (SSL *) SSL_new(imclient->tls_ctx);
