@@ -1,6 +1,6 @@
 /* script.c -- sieve script functions
  * Larry Greenfield
- * $Id: script.c,v 1.28 2000/05/28 22:45:59 leg Exp $
+ * $Id: script.c,v 1.35 2000/07/25 17:00:45 ken3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -226,13 +226,18 @@ static int look_for_me(char *myaddr, stringlist_t *myaddrs, const char **body)
 		found = 1;
 		break;
 	    }
-	    
-	    for (sl = myaddrs; sl != NULL; sl = sl->next) {
+
+	    for (sl = myaddrs; sl != NULL && !found; sl = sl->next) {
+		void *altdata = NULL, *altmarker = NULL;
+		char *altaddr;
+
 		/* is this address one of my addresses? */
-		if (!strcmp(addr, sl->s)) {
+		parse_address(sl->s, &altdata, &altmarker);
+		altaddr = get_address(ADDRESS_ALL, &altdata, &altmarker);
+		if (!strcmp(addr, altaddr))
 		    found = 1;
-		    break;
-		}
+
+		free_address(&altdata, &altmarker);
 	    }
 	}
 	free_address(&data, &marker);
@@ -352,7 +357,7 @@ static int evaltest(sieve_interp_t *i, test_t *t, void *m)
 }
 
 /* evaluate the script c.  returns negative if error was encountered,
-   0 if it exited off the end, or if a stop action was encountered.
+   0 if it exited off the end, or positive if a stop action was encountered.
 
    note that this is very stack hungry; we just evaluate the AST in
    the naivest way.  if we implement some sort of depth limit, we'll
@@ -474,6 +479,10 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 				   (i->getheader(m, buf, &body) == SIEVE_OK)))
 			found = look_for_me(myaddr, c->u.v.addresses, body);
 
+		    if (!found && (strcpy(buf, "bcc"),
+				   (i->getheader(m, buf, &body) == SIEVE_OK)))
+			found = look_for_me(myaddr, c->u.v.addresses, body);
+
 		    if (!found)
 			l = SIEVE_DONE;
 		}
@@ -520,9 +529,8 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 		}
 		break;
 	    }
-	case STOP: /* we're done, exit */
+	case STOP:
 	    res = 1;
-	    return 0;
 	    break;
 	case DISCARD:
 	    res = do_discard(actions);
@@ -570,7 +578,7 @@ static int eval(sieve_interp_t *i, commandlist_t *c,
 
 	}
 
-	if (res) /* we've either encountered an error */
+	if (res) /* we've either encountered an error or a stop */
 	    break;
 
 	/* execute next command */
@@ -783,7 +791,7 @@ static int makehash(unsigned char hash[HASHSIZE], char *s1, char *s2)
 int sieve_execute_script(sieve_script_t *s, void *message_context)
 {
     int ret = 0;
-    int implicit_keep;
+    int implicit_keep = 0;
     action_list_t *actions = NULL, *a;
     action_t lastaction = -1;
     notify_action_t *notify_action;
@@ -800,8 +808,8 @@ int sieve_execute_script(sieve_script_t *s, void *message_context)
 	goto error;
     }
  
-    if ((ret = eval(&s->interp, s->cmds, message_context, actions,
-		    notify_action, &errmsg)) != SIEVE_OK)
+    if (eval(&s->interp, s->cmds, message_context, actions,
+	     notify_action, &errmsg) < 0)
 	return SIEVE_RUN_ERROR;
   
     strcpy(actions_string,"Action(s) taken:\n");
@@ -980,18 +988,6 @@ int sieve_execute_script(sieve_script_t *s, void *message_context)
 	}
     }
 
-    if (implicit_keep) {
-	sieve_keep_context_t keep_context = { &s->interp.curflags };
- 
-	lastaction = ACTION_KEEP;
-        ret = s->interp.keep(&keep_context, s->interp.interp_context,
-			     s->script_context, message_context, &errmsg);
-        if (ret == SIEVE_OK)
-            snprintf(actions_string+strlen(actions_string),
-		     sizeof(actions_string)-strlen(actions_string),
-		     "Kept\n");
-    }
-
  error: /* report run-time errors */
  
     if (ret != SIEVE_OK) {
@@ -1025,7 +1021,27 @@ int sieve_execute_script(sieve_script_t *s, void *message_context)
 	ret |= s->interp.execute_err(buf, s->interp.interp_context,
 				     s->script_context, message_context);
     }
-  
+
+    if (implicit_keep) {
+	sieve_keep_context_t keep_context;
+	int keep_ret;
+
+	keep_context.imapflags = &s->interp.curflags;
+ 
+	lastaction = ACTION_KEEP;
+	keep_ret = s->interp.keep(&keep_context, s->interp.interp_context,
+			     s->script_context, message_context, &errmsg);
+	ret |= keep_ret;
+        if (keep_ret == SIEVE_OK)
+            snprintf(actions_string+strlen(actions_string),
+		     sizeof(actions_string)-strlen(actions_string),
+		     "Kept\n");
+	else {
+	    implicit_keep = 0;	/* don't try an implicit keep again */
+	    goto error;		/* process the implicit keep error */
+	}
+    }
+
     if (actions)
 	free_action_list(actions);
   
